

Intel(R) Software Guard Extensions Multi-package Registration Agent for Linux and Windows
================================================

Introduction
------------
The SGX Multi-Package Registration Agent is a software layer between the BIOS 
and the SGX Registration Server which manages the SGX multi-package 
registration flows. SGX Multi-Package Registration Agent is responsible 
to deliver requests from BIOS to the SGX Registration Server.

Platform registration requests are triggered via UEFI variables and delivered
to registration server by the agent over a TLS connection.

Any pending registration request is handled by the agent upon OS boot, if there
is no pending request the agent stops till next OS boot.

License
-------
See [license.txt](license.txt) for details.

Documentation
-------------
See [doc/README](doc/README) for details.

Build the Intel(R) SGX Multi-package Registration Agent and Libraries
------------------------
### Prerequisites
- Ensure that you have one of the following operating systems:  
  * Red Hat Enterprise Linux Server release 7.6 64bits
  * Red Hat Enterprise Linux Server release 8.0 64bits
  * Ubuntu Server 16.04
  * Ubuntu Server 18.04
  * Microsoft Windows Server 2019 RS5

- Linux Prerequisites list:
  * SGX-SDK
  * libcurl
  * For Red Hat Enterprise Linux Server release
```
    $ sudo yum groupinstall 'Development Tools'
    $ sudo yum install rpm-build zip
```
  * For Ubuntu Server release
```
    $ sudo apt-get install build-essential debhelper zip
```
  
- Windows Prerequisites list:
  * Visual Studio 2019
  * SGX-SDK

### Build Intel(R) SGX Multi-package Registration Agent, libraries, tool and installer in Linux

- To build Intel(R) SGX Multi-package Registration Agent, libraries, tool with default configuration, enter the following command:
```
  $ make 
```
You can find the tools and libraries generated in the `build` directory.
- To build Intel(R) SGX Multi-package Registration Agent, libraries, tool  with debug information, enter the following command:
```
  $ make DEBUG=1
```
- To clean the files generated by previous `make` command, enter the following command:
```
  $ make clean
```
- To build the Intel(R) SGX Multi-package Registration Agent installer, enter the following command:
  * On Ubuntu 16.04 and Ubuntu 18.04:
   ```
  $ make deb_pkg
  ```
  You can find the generated Intel(R) SGX Multi-package Registration Agent installers located under `build/installer`.

  **Note**: On Ubuntu 18.04, besides the Intel(R) SGX Multi-package Registration Agent installer, the above command generates another debug symbol package named ``package-name-dbgsym_${version}-${revision}_amd64.ddeb`` for debug purpose. On Ubuntu 16.04, if you want to keep debug symbols in the Intel(R) SGX Multi-package Registration Agent installer, before building the Intel(R) SGX Multi-package Registration Agent installer, you need to export an environment variable to ensure the debug symbols not stripped:
   ```
   $ export DEB_BUILD_OPTIONS="nostrip"
   ```
  **Note**: The above command builds the Intel(R) SGX Multi-package Registration Agent with default configuration firstly and then generates the target Multi-package Registration Agent Installer. To build the Intel(R) SGX Multi-package Registration Agent Installer without optimization and with full debug information kept in the tools and libraries, enter the following command:
  ```
  $ make deb_pkg DEBUG=1
  ```
  * On Red Hat Enterprise Linux 7.4, Red Hat Enterprise Linux 8.0:
  ```
  $ make rpm_pkg
  ```
  You can find the generated Intel(R) SGX Multi-package Registration Agent installers located under `build/installer`.

  **Note**: The above command builds the Intel(R) SGX Multi-package Registration Agent with default configuration firstly and then generates the target Multi-package Registration Agent Installer. To build the Intel(R) SGX Multi-package Registration Agent Installer with debug information kept in the tools and libraries, enter the following command:

  ```
  $ make rpm_pkg DEBUG=1
  ```

### Build Intel(R) SGX Multi-package Registration Agent installer in Windows
- Open solution and build Release configuration.
- Binaries will be found under "x64" directory.
- Once build completed, generate the inf installer using the commands below, INF installation package will be found under: "inf/sgx_ra_<version>"

```
  $ cd inf
  $ build_inf_installer.cmd
```

### Install the Intel(R) Multi-package Registration Agent in Linux
To install the registration agent, follow the instructions:

  * Installation path:
```
  $ /opt/intel/sgx-ra-service
```

  * Installation instructions:
```
    RHEL:
  $ sudo yum install libsgx-ra-network-<version>-1.el8.x86_64.rpm
  $ sudo yum install libsgx-ra-uefi-<version>-1.el8.x86_64.rpm
  $ sudo yum install sgx-ra-service-<version>-1.el8.x86_64.rpm

    Ubuntu:
  $ sudo dpkg -i libsgx-ra-network_{version}-{revision}_{arch}.deb
  $ sudo dpkg -i libsgx-ra-uefi_{version}-{revision}_{arch}.deb
  $ sudo dpkg -i sgx-ra-service_{version}-{revision}_{arch}.deb
```

  * Uninstallation instructions:
```
    RHEL:
  $ sudo yum remove sgx-ra-service.x86_64
  $ sudo yum remove libsgx-ra-network.x86_64
  $ sudo yum remove libsgx-ra-uefi.x86_64

    Ubuntu:
  $ sudo dpkg -r sgx-ra-service
  $ sudo dpkg -r libsgx-ra-network
  $ sudo dpkg -r libsgx-ra-uefi
```

  * Log file:
```
  $ /var/log/mpa_registration.log
```
        
  * The agent supports an optional configuration file, located in:
```
  $ /etc/mpa_registration.conf 
```

### Install the Intel(R) Multi-package Registration Agent in Windows

To install the registration agent, follow the instructions:
  * Navigate to "Device Manager"->"Software Components"->"Intel(R) Software Guard Extensions Software Multi-Package Registration"

Install SGX Multi-package Registration Agent driver:
  * Navigate to "Device Manager"->"Software Components"->"Intel(R) Software Guard Extensions Software Multi-Package Registration"
  * Right click on "Intel(R) Software Guard Extensions Software Multi-Package Registration"
  * Choose "Update driver"
  * On the "Update drivers" windows choose "Browse my computer for driver software".
  * Navigate to "sgx_mpa" directory in the installation package.
  * Click next and approve all dialogs during installation.
  * Once installation is finished, the agent is fully installed on your system.


### Start or Stop registration service in Linux
The Intel(R) SGX Multi-package Registration agent installer installs a registration agent service in your machine. 

To stop the service: `$ sudo systemctl stop mpa_registration_tool.service`  
To start the service: `$  sudo systemctl start mpa_registration_tool.service`  
To restart the service: `$  sudo systemctl restart mpa_registration_tool.service`

### Start or Stop registration service in Windows
The Intel(R) SGX Multi-package Registration agent installer installs an mpa service in your machine. 

To stop/start/restart the service, navigate to "Services" and look for "Intel(R) SGX Multi-Package Registration Service".

### Optional configuration file in Linux
- The agent supports an optional configuration file, located in:
```
  $ /etc/mpa_registration.conf 
```
  * Subscription key setting:
```
        To set the subscription key, add the following line:
            subscription key = <64byte-hex-value>
        e.g:
            subscription key = 7a963d696ff94b7d82df4cbe924b1574
```
        
  * Proxy settings:
```
        To set default proxy configuration, as configured in your operation system environment (HTTPS_PROXY):
            proxy type = default
        To set direct access to the Internet without any proxy configuration:
            proxy type = direct
        To set manual proxy setting, which also supports authenticated proxy 
         configuration:
            proxy type = manual
            proxy url = proxy-chain.intel.com:912
        The format of url is standard: 
            user:password@proxy:port
```

  * UEFI path (linux only):
```
        To set uefi path (default path = /sys/firmware/efi/efivars):
            uefi path = /home/user/uefi
```
                
  *  Log level:
```
        To set logging level:
            No logs:
                log level = none
            Functional, general logs:
                log level = func
            Default, general logs and errors:
                log level = error
            Verbose, general logs, errors and verbose:
                log level = info
```

### Optional configurations in Windows
- The agent supports an optional configurations using some registry keys:

  * Subscription key setting: HKEY_LOCAL_MACHINE\SOFTWARE\Intel\SGX_RA\RASubscriptionKey
```
        To set the subscription key, add the following String key name "token":
            <64byte-hex-value>
        e.g:
            7a963d696ff94b7d82df4cbe924b1574
```
  * Proxy settings: HKEY_LOCAL_MACHINE\SOFTWARE\Intel\SGX_RA\RAProxy
```
        To set default proxy configuration, as configured in your operation system, add the following DWORD key name "type":
            "0"
        To set direct access to the Internet without any proxy configuration, add the following DWORD key name "type":
            "1"
        To set manual proxy setting, which also supports authenticated proxy configuration, add the following DWORD key name "type":
            "2"
        To set the specific proxy URL, add the following String key "url":
            e.g: "proxy-chain.intel.com:912"
        The format of url is standard: 
            user:password@proxy:port
```      
  *  Log level settings: HKEY_LOCAL_MACHINE\SOFTWARE\Intel\SGX_RA\RALog
```
        To set logging level, add the following DWORD key "level":
            No logs:
                "0"
            Functional, general logs:
                "1"
            Default, general logs and errors:
                "2"
            Verbose, general logs, errors and verbose:
                "3"
```
